/*

	DFApplication.cpp
	
	Copyright 1996 Thorsten Seitz, All Rights Reserved
	
*/

#include "DFApplication.h"


// class DFActivationWindow

#pragma mark ## ACTIVATION_WINDOW

DFActivationWindow::DFActivationWindow(void)
	: BWindow(BRect(-10,-10,-9,-9), "", B_TITLED_WINDOW, 0)
{
	fAppActive = TRUE;
}


void
DFActivationWindow::WindowActivated(bool flag)
{
	if (flag) {
		if (!fAppActive) {
			be_app->Activate();
		}
		else {
			Activate(FALSE);
		}
	}	
}


void
DFActivationWindow::SetAppActive(bool flag)
{
	fAppActive = flag;
}


// class DFApplication

#pragma mark ## APPLICATION

DFApplication::DFApplication(ulong signature)
	: BApplication(signature), hidden(FALSE), windowList(5)
{
	AddStandardMenuItems();
	
	fActivationWindow = new DFActivationWindow();
	
	// try to start the miniWindowDaemon
	
	fMWD = FALSE;
	fMW = FALSE;
	long result = be_roster->Launch('?MWD');
	
	switch (result) {
	
	case B_BAD_VALUE:
	case B_ERROR:
	case B_LAUNCH_FAILED:
		long tryMW = (new BAlert("MiniWindowDaemon launch failure",
		"The MiniWindowDaemon could not be launched on your system.\n"
		"If you want to use mini windows instead of title tabs "
		"for minimized windows, it is needed to administrate "
		"their arrangement on your desktop.\nMini windows can display "
		"status informations for their respective windows."
		"If you would like to use this feature, please get "
		"MiniWindowDaemon from BeWare.\n"
		"You may try to use mini windows nevertheless:\n"
		"In this case each mini window is put on the upper "
		"left of your desktop just to the right of the dock "
		"the first time it is created.", 
		"Try mini windows",
		"Use normal minimization", NULL,
		B_WIDTH_FROM_LABEL, B_WARNING_ALERT))->Go();
		if (tryMW == 0) fMW = TRUE;
		break;
		
	case B_NO_ERROR:
	case B_ALREADY_RUNNING:
		fMWD = TRUE;
		fMW = TRUE;
		BMessenger mwd('?MWD');
		mwd.SendMessage(DF_ADD);
		break;
		
	default:
		break;
	}
}


DFApplication::~DFApplication()
{
	if (fMWD) {
		BMessenger mwd('?MWD');
		if (mwd.Error() == B_NO_ERROR)
			mwd.SendMessage(DF_QUIT);
	}
}


void
DFApplication::AddStandardMenuItems(void)
{
	BMenuItem  *item;
	BPopUpMenu *menu = MainMenu();
	
	MainMenu()->AddItem(new BSeparatorItem(), 1);	
	
	item = new BMenuItem("Lower", new BMessage(DF_LOWER));
	item->SetTarget(be_app);
	menu->AddItem(item, 2);

	item = new BMenuItem("Raise", new BMessage(DF_RAISE));
	item->SetTarget(be_app);
	menu->AddItem(item, 3);
	
	item = new BMenuItem("Hide", new BMessage(DF_HIDE));
	item->SetTarget(be_app);
	menu->AddItem(item, 4);
	
	item = new BMenuItem("Hide all others", 
				new BMessage(DF_HIDE_ALL_OTHERS));
	item->SetTarget(be_app);
	menu->AddItem(item, 5);	
	
	MainMenu()->AddItem(new BSeparatorItem(), 6);	
}	


void
DFApplication::Activate(void)
{
	DFWindow				*window;
	long     				i=0;
	bool					success = FALSE;
	StLocker<BApplication> 	lock(this);

	while( window = static_cast<DFWindow*>(windowList.ItemAt(i++)) )
		if (!window->IsMinimized() && !window->IsHidden()) {
			window->Activate(TRUE);
			success = TRUE;
			break;
		}
	
	if (!success) {
		window = static_cast<DFWindow*>(windowList.ItemAt(0));
		if (window->IsMinimized())
			window->Minimize(FALSE);
		if (window->IsHidden())
			window->Show();
	}
}


void
DFApplication::AppActivated(bool flag)
{
	if (flag && hidden) Hide(FALSE);
	if (flag) Activate();
	fActivationWindow->SetAppActive(flag);
}


void
DFApplication::MessageReceived(BMessage *message)
{
	switch (message->what) {
	
	case DF_LOWER:
		Raise(FALSE);
		break;
		
	case DF_RAISE:
		Raise(TRUE);
		break;
		
	case DF_HIDE:
		Hide(TRUE);
		break;
		
	case DF_UNHIDE:
		Hide(FALSE);
		break;
		
	case DF_HIDE_ALL_OTHERS:
		HideAllOthers();
		break;
		
	default:
		inherited::MessageReceived(message);
		break;
	}
}


void
DFApplication::Raise(bool raise)
{
	DFWindow 				*window;
	long     				i=0;
	StLocker<BApplication> 	lock(this);

	while( window = static_cast<DFWindow*>(windowList.ItemAt(i++)) )
		window->Activate(raise);
}
	

void
DFApplication::Hide(bool hide)
{
	DFWindow   				*window;
	long       				i=0;
	StLocker<BApplication> 	lock(this);

	if (hidden == hide) return;
	
	hidden = hide;
	while( window = static_cast<DFWindow*>(windowList.ItemAt(i++)) )
		if (hide)
			window->Hide();
		else
			window->Show();
}


void
DFApplication::HideAllOthers(void)
{
	team_id  who, me = Team();
	long     i;
	BList    teams;
	app_info appInfo;
	
	
	be_roster->GetAppList(&teams);
	
	for (i=0; i<teams.CountItems(); i++) {
		who = static_cast<team_id>(teams.ItemAt(i));
		if (who != me) {
			be_roster->GetRunningAppInfo(who, &appInfo);
			BMessenger hermes(appInfo.signature, who);
			hermes.SendMessage(DF_HIDE);
		}
	}
}


void
DFApplication::RegisterWindow(DFWindow *window)
{
	StLocker<BApplication> lock(this);

	windowList.AddItem(window);
	long i = windowList.IndexOf(window);
}


void
DFApplication::UnregisterWindow(DFWindow *window)
{
	StLocker<BApplication> lock(this);

	long i = windowList.IndexOf(window);
	windowList.RemoveItem(static_cast<void*>(window));
	if (windowList.IsEmpty())
		be_app->PostMessage(B_QUIT_REQUESTED);
}


void
DFApplication::WindowActivated(DFWindow *window)
{
	StLocker<BApplication> lock(this);

	windowList.RemoveItem(static_cast<void*>(window));
	windowList.AddItem(static_cast<void*>(window), 0);
}
	